--- milter-ahead.c.orig	2004-09-06 08:38:55.000000000 +0000
+++ milter-ahead.c	2019-07-04 15:50:56.907064000 +0000
@@ -491,6 +491,7 @@
 	workspace data;
 	CacheEntry entry;
 	char *error, *auth_authen, *rcpt_addr, *rcpt_host, *rcpt_mailer, *helo;
+	char *s, *tofree;
 
 	rc = SMFIS_CONTINUE;
 
@@ -580,18 +581,38 @@
 	if (*data->work.rcpt->localLeft.string == '\0')
 		return smfReply(&data->work, 553, "5.1.0", "cannot deliver to null address");
 
-	if (smfLogDetail & SMF_LOG_DIALOG)
-		syslog(LOG_DEBUG, TAG_FORMAT "contacting " HOP_INFO_FORMAT, TAG_ARGS, HOP_INFO_ARGS);
+	if (forceNextHop == NULL) {
+		tofree = s = strdup(rcpt_host); /* Copy for strsep() */
+		if (tofree == NULL) {
+			rc = smfReply(&data->work, 450, "4.7.1", "no enough memory");
+			goto error0;
+		}
+	} else
+		tofree = NULL;
+	data->server = NULL;
+
+    /* Loop over possible list [relay1]:[relay2]:[relay3] where relayN may be %0 or %1 */
+    while (data->server == NULL) {
+	if (forceNextHop == NULL && (rcpt_host = strsep(&s, ":")) == NULL)
+		break;
 
 	/* Connect to next hop. */
 	if (forceNextHop == NULL) {
 		(void) sscanf(rcpt_host, PARSE_MAILER_HOST, data->line);
+		if (data->line[0] == '%')
+			continue;
 	} else {
 		(void) snprintf(data->line, sizeof (data->line), "%s", forceNextHop);
 		rcpt_host = forceNextHop;
 	}
 	
+	if (smfLogDetail & SMF_LOG_DIALOG)
+		syslog(LOG_DEBUG, TAG_FORMAT "contacting " HOP_INFO_FORMAT, TAG_ARGS, HOP_INFO_ARGS);
+
 	data->server = SocketOpenTcpClientWait(data->line, SMTP_PORT, socketTimeout);
+	if (forceNextHop != NULL && data->server == NULL)
+		break;
+    }
 
 	if (data->server == NULL) {
 		if (isBackupMx)
@@ -629,9 +650,9 @@
 	 */
 	status = 450;
 
-	if ((helo = smfi_getsymval(ctx, "{if_name}")) == NULL
-	&&  (helo = smfi_getsymval(ctx, "{if_addr}")) == NULL
-	&&  (helo = smfi_getsymval(ctx, "j")) == NULL)
+	if ((helo = smfi_getsymval(ctx, "j")) == NULL
+	&&  (helo = smfi_getsymval(ctx, "{if_name}")) == NULL
+	&&  (helo = smfi_getsymval(ctx, "{if_addr}")) == NULL)
 		helo = "127.0.0.1";
 		
 	(void) snprintf(data->line, sizeof (data->line), "HELO %s\r\n", helo);
@@ -711,6 +732,8 @@
 		data->server = NULL;
 	}
 error0:	
+	if (tofree != NULL)
+		free(tofree);
 	return rc;
 }
 
